/**
 * *****************************************************************************
 * This application has been revised as a part of an Interactive Qualifying
 * Project for Worcester Polytechnic Institute and our sponsor, SHPI Ingenieria.
 *
 * Revision Developer: Kyle Fortin
 *
 * Revision Authors: Stella Banou, Devin Duarte, Kyle Fortin, Taylor Llodra
 *
 * The application is based on work completed by a previous IQP team from WPI.
 *
 * Original Developer: Tyler Stone
 *
 * Original Authors: Alexandra Hardin, Eric Schattschneider, Tyler Stone,
 * Kristina Walker
 * ****************************************************************************
 */
package sa.shpi.advanced.pdf;

import java.io.FileOutputStream;
import java.util.ArrayList;
import java.util.Date;

import sa.shpi.advanced.model.Warehouse;
import sa.shpi.advanced.model.content.FurnishingList;
import sa.shpi.advanced.model.content.Room;
import sa.shpi.advanced.model.structure.MaterialList;

import com.itextpdf.text.Document;
import com.itextpdf.text.DocumentException;
import com.itextpdf.text.Element;
import com.itextpdf.text.Font;
import com.itextpdf.text.Paragraph;
import com.itextpdf.text.Phrase;
import com.itextpdf.text.pdf.PdfPCell;
import com.itextpdf.text.pdf.PdfPTable;
import com.itextpdf.text.pdf.PdfWriter;
import java.awt.Desktop;
import java.io.File;
import java.io.IOException;
import java.time.Instant;
import sa.shpi.advanced.model.shelf.Shelf;
import sa.shpi.advanced.model.shelf.ShelfSubType;

/**
 * This class automatically generates a PDF report upon instantiation.
 *
 * @author Kyle
 *
 */
public class PdfGenerator {

    private static long date = new Date().getTime();
    /**
     * First text string denotes the file path to be used. Second text string
     * denotes file extension.
     */
    private static String filename = System.getProperty("user.home") + "/Documents/SHPI Reportes de Emisiones/"+ date + ".pdf";
    private static File dir = new File(System.getProperty("user.home") + "/Documents/SHPI Reportes de Emisiones");
    private static File pdf = new File(filename);
    private static Font catFont = new Font(Font.FontFamily.TIMES_ROMAN, 18,
            Font.BOLD);
    private static Font subFont = new Font(Font.FontFamily.TIMES_ROMAN, 16,
            Font.BOLD);
    private static Font smallBold = new Font(Font.FontFamily.TIMES_ROMAN, 12,
            Font.BOLD);

    private static Warehouse ware;

    public PdfGenerator(Warehouse ware) {
        PdfGenerator.ware = ware;

        if (dir.exists()) {
            try {
                Document document = new Document();
                PdfWriter.getInstance(document, new FileOutputStream(filename));
                document.open();
                addTitlePage(document);
                addContent(document);
                document.close();
                if (Desktop.isDesktopSupported()) {
                    try {
                        Desktop.getDesktop().open(pdf);
                    } catch (IOException ex) {
                        //No application registered for .PDF ...
                    }
                }
            } catch (Exception e) {
                e.printStackTrace();
            }
        } else {
            try {
                dir.mkdir();
                Document document = new Document();
                PdfWriter.getInstance(document, new FileOutputStream(filename));
                document.open();
                addTitlePage(document);
                addContent(document);
                document.close();
                if (Desktop.isDesktopSupported()) {
                    try {
                        Desktop.getDesktop().open(pdf);
                    } catch (IOException ex) {
                        //No application registered for .PDF ...
                    }
                }
            } catch (Exception e) {
                e.printStackTrace();
            }
        }
    }

    private static void addTitlePage(Document document)
            throws DocumentException {

        Paragraph preface = new Paragraph();
        addEmptyLine(preface, 1);

        // Write the header.
        preface.add(new Paragraph(
                "Reporte de Emisiones - Bodega",
                catFont));
        preface.add(new Paragraph("SHPI Ingenieria S.A.", smallBold));
        addEmptyLine(preface, 1);

        preface.add(new Paragraph("Reporte generado: " + new Date(), smallBold));
        addEmptyLine(preface, 3);

        document.add(preface);
    }

    private static void addContent(Document document) throws DocumentException {
        //Basic details of fire.
        Paragraph wareDetails = new Paragraph("Detalles del Incendio:", subFont);
        wareDetails.add(new Paragraph("Área total de la bodega: " + String.format("%.2f", ware.getAreaTotal())
                + " m^2"));
        wareDetails.add(new Paragraph("Área quemada: " + String.format("%.2f",ware.getAreaBurned())
                + " m^2"));
        addEmptyLine(wareDetails, 2);

        Paragraph totalEmissions = new Paragraph("Total de Emisiones: "
                + String.format("%.2f kg CO2", ware.getTotalEmissions()),
                catFont);

        //Add to document:
        document.add(wareDetails);
        document.add(generateStructureSection());
        document.add(generateContentSection());
        document.add(generateShelfSection());
        document.add(totalEmissions);
    }

    private static Paragraph generateStructureSection() {
        //Add structural details:
        Paragraph structureDetails = new Paragraph("La Estructura ("
                + String.format("%.2f kg CO2", ware.getStructureEmissions())
                + "):", subFont);
        PdfPTable structureTable = new PdfPTable(3);

        PdfPCell c1 = new PdfPCell(new Phrase("Categoría"));
        c1.setHorizontalAlignment(Element.ALIGN_CENTER);
        structureTable.addCell(c1);

        c1 = new PdfPCell(new Phrase("Emisiones de Categoría"));
        c1.setHorizontalAlignment(Element.ALIGN_CENTER);
        structureTable.addCell(c1);
        
        c1 = new PdfPCell(new Phrase("El Material"));
        c1.setHorizontalAlignment(Element.ALIGN_CENTER);
        structureTable.addCell(c1);

        //Complete table dynamically:
        ArrayList<sa.shpi.advanced.model.structure.Section> wareSections = ware
                .getSections();
        for (int count = 0; count < wareSections.size(); count++) {
            structureTable.addCell(wareSections.get(count).getSectionCategory()
                    .toString());
            
            structureTable.addCell(String.format("%.2f", wareSections.get(count).calculateEmissions(ware.getAreaBurned())));

            MaterialList matList = wareSections.get(count).getMaterialList();
            for (int c2 = 0; c2 < matList.size(); c2++) {
                structureTable.addCell(matList.get(c2).getMaterial().getName());
                structureTable.addCell("");
                structureTable.addCell("");
            }
            //Add blank cells to complete row:
            structureTable.addCell("");
            //Delete empty row:
            structureTable.deleteLastRow();
        }

        structureDetails.add(structureTable);

        return structureDetails;
    }

    private static Paragraph generateContentSection() {
        //Add conent details:
        Paragraph contentDetails = new Paragraph("Los Áreas ("
                + String.format("%.2f kg CO2", ware.getContentEmissions())
                + "):", subFont);
        PdfPTable contentTable = new PdfPTable(5);

        PdfPCell c1 = new PdfPCell(new Phrase("Tipo de Área"));
        c1.setHorizontalAlignment(Element.ALIGN_CENTER);
        contentTable.addCell(c1);
        
        c1 = new PdfPCell(new Phrase("Cantidad de Área"));
        c1.setHorizontalAlignment(Element.ALIGN_CENTER);
        contentTable.addCell(c1);

        c1 = new PdfPCell(new Phrase("Emisiones de Área"));
        c1.setHorizontalAlignment(Element.ALIGN_CENTER);
        contentTable.addCell(c1);

        c1 = new PdfPCell(new Phrase("Contenido"));
        c1.setHorizontalAlignment(Element.ALIGN_CENTER);
        contentTable.addCell(c1);
        
        c1 = new PdfPCell(new Phrase ("Cantidad de Contenido"));
        c1.setHorizontalAlignment(Element.ALIGN_CENTER);
        contentTable.addCell(c1);

        ArrayList<Room> rooms = ware.getRooms();
        for (int count = 0; count < rooms.size(); count++) {
            contentTable.addCell(rooms.get(count).getRoomType().toString());
            
            contentTable.addCell(String.valueOf(rooms.get(count).getRoomQuantity()));
            
            contentTable.addCell(String.valueOf(String.format("%.2f", rooms.get(count).calculateEmissions()) + " kg CO2"));
            
            FurnishingList furnishingList = rooms.get(count)
                    .getFurnishingList();
            
            for (int c2 = 0; c2 < furnishingList.size(); c2++) {
                contentTable.addCell(furnishingList.get(c2).getFurnishing()
                        .getName());
                contentTable.addCell(String.valueOf(furnishingList.get(c2)
                        .getQuantity()));
                contentTable.addCell("");
                contentTable.addCell("");
                contentTable.addCell("");
            
                }
            
            //Add blank cells to complete the row.
            
            contentTable.addCell("");
            contentTable.addCell("");

            //Delete empty row.
            contentTable.deleteLastRow();
        }

        contentDetails.add(contentTable);

        return contentDetails;
    }

    private static Paragraph generateShelfSection() {
        //Add conent details:
        Paragraph shelfDetails = new Paragraph("Los Estantes ("
                + String.format("%.2f kg CO2", ware.getShelfEmissions())
                + "):", subFont);
        PdfPTable contentTable = new PdfPTable(3);

        PdfPCell c1 = new PdfPCell(new Phrase("El Tipo"));
        c1.setHorizontalAlignment(Element.ALIGN_CENTER);
        contentTable.addCell(c1);

        c1 = new PdfPCell(new Phrase("Cantidad de Estante"));
        c1.setHorizontalAlignment(Element.ALIGN_CENTER);
        contentTable.addCell(c1);
        
        c1 = new PdfPCell(new Phrase("Emisiones de Estante"));
        c1.setHorizontalAlignment(Element.ALIGN_CENTER);
        contentTable.addCell(c1);

        ArrayList<Shelf> shelves = ware.getShelves();
        for (int count = 0; count < shelves.size(); count++) {
            
            ShelfSubType shelfsubtype = shelves.get(count)
                    .getShelfSubType();
            
            contentTable.addCell(shelfsubtype.toString());
            
            contentTable.addCell(String.valueOf(shelves.get(count).getShelfQuantity()));
            
            contentTable.addCell(String.format("%.2f", shelves.get(count).calculateEmissions()) + " kg CO2");
            
            contentTable.addCell("");
            //Add blank cells to complete the row.
            contentTable.addCell("");
            contentTable.addCell("");
            //Delete empty row.
            contentTable.deleteLastRow();
        }

        shelfDetails.add(contentTable);

        return shelfDetails;
    }    
    
    private static void addEmptyLine(Paragraph paragraph, int number) {
        for (int i = 0; i < number; i++) {
            paragraph.add(new Paragraph(" "));
        }
    }

}
