/**
 * *****************************************************************************
 * This application has been revised as a part of an Interactive Qualifying
 * Project for Worcester Polytechnic Institute and our sponsor, SHPI Ingenieria.
 *
 * Revision Developer: Kyle Fortin
 *
 * Revision Authors: Stella Banou, Devin Duarte, Kyle Fortin, Taylor Llodra
 *
 * The application is based on work completed by a previous IQP team from WPI.
 *
 * Original Developer: Tyler Stone
 *
 * Original Authors: Alexandra Hardin, Eric Schattschneider, Tyler Stone,
 * Kristina Walker
 * ****************************************************************************
 */
package sa.shpi.advanced.model;

import java.util.ArrayList;
import sa.shpi.advanced.model.content.FurnishingLibrary;

import sa.shpi.advanced.model.content.Room;
import sa.shpi.advanced.model.structure.ExtWallsMaterialsLibrary;
import sa.shpi.advanced.model.structure.FlooringMaterialsLibrary;
import sa.shpi.advanced.model.structure.MaterialsLibrary;
import sa.shpi.advanced.model.structure.RoofingMaterialsLibrary;
import sa.shpi.advanced.model.structure.Section;
import sa.shpi.advanced.model.structure.SectionCategory;

/**
 * Class for the industrial building. This class stores all information about
 * the building and its components. It also stores a running total for the area
 * of all rooms added to the building.
 *
 * @author Kyle
 *
 */
public class Industrial {

    private String name;
    private ArrayList<Room> rooms;
    private Section externalWalls = new Section(SectionCategory.EXTERNAL_WALLS);
    private Section flooring = new Section(SectionCategory.FLOORING);
    private Section roofing = new Section(SectionCategory.ROOFING);
    private SectionCategory sectionCat;

    private static double areaTotal;
    private static double areaBurned;
    private static IndustrialType industrialType;
    private MaterialsLibrary materialsLibrary;
    private FurnishingLibrary furnishingsLibrary;
    public static double totalRoomArea;

    public void setIndSectionCat(SectionCategory sectioncat) {
        this.sectionCat = sectioncat;
    }
    
    public static void addTotalRoomArea(double integer) {
        totalRoomArea += integer;
    }

    public void setFurnishingsLibrary(FurnishingLibrary furnishingsLibrary) {
        this.furnishingsLibrary = furnishingsLibrary;
    }

    public FurnishingLibrary getFurnishingsLibrary() {
        return furnishingsLibrary;
    }

    public MaterialsLibrary getMaterialsLibrary() {
        return materialsLibrary;
    }

    public void setMaterialsLibrary(MaterialsLibrary materialsLibrary) {
        this.materialsLibrary = materialsLibrary;
    }

    public void initializestructurelibrary() {
        //Initialize library corresponding to selected section.
        if (sectionCat.equals(SectionCategory.EXTERNAL_WALLS)) {
            setMaterialsLibrary(new ExtWallsMaterialsLibrary());
        } else if (sectionCat.equals(SectionCategory.FLOORING)) {
            setMaterialsLibrary(new FlooringMaterialsLibrary());
        } else if (sectionCat.equals(SectionCategory.ROOFING)) {
            setMaterialsLibrary(new RoofingMaterialsLibrary());
        }
    }
    
    public void initializeroomlibrary() {
        setFurnishingsLibrary(new FurnishingLibrary());
    }

    public Industrial() {
        name = Strings.INDUSTRIAL.toString();
        rooms = new ArrayList<Room>();
    }

    public Industrial(String name) {
        this.name = name;
        rooms = new ArrayList<Room>();
    }

    public ArrayList<Room> getRooms() {
        return rooms;
    }

    public void setRooms(ArrayList<Room> rooms) {
        this.rooms = rooms;
    }

    public void addRoom(Room room) {
        rooms.add(room);
    }

    public Section getExternalWalls() {
        return externalWalls;
    }

    public void setExternalWalls(Section externalWalls) {
        this.externalWalls = externalWalls;
    }

    public Section getFlooring() {
        return flooring;
    }

    public void setFlooring(Section flooring) {
        this.flooring = flooring;
    }

    public Section getRoofing() {
        return roofing;
    }

    public void setRoofing(Section roofing) {
        this.roofing = roofing;
    }

    public static double getAreaTotal() {
        return areaTotal;
    }

    public void setAreaTotal(double areaTotal) {
        Industrial.areaTotal = areaTotal;
    }

    public static double getAreaBurned() {
        return areaBurned;
    }

    public void setAreaBurned(double areaBurned) {
        this.areaBurned = areaBurned;
    }

    public static void setIndustrialType(IndustrialType industrialtype) {
        Industrial.industrialType = industrialtype;
    }

    public static IndustrialType getIndustrialType() {
        return industrialType;
    }

    public double getTotalEmissions() {
        return getContentEmissions() + getStructureEmissions();
    }

    public double getContentEmissions() {
        double emissions = 0.0;

        //For each room:
        for (int count = 0; count < rooms.size(); count++) {
            //Add the emissions from the room together.
            emissions += rooms.get(count).calculateEmissions();
        }

        return emissions;
    }

    public double getStructureEmissions() {
        double emissions = 0.0;

        //For each structural component:
        for (int count = 0; count < getSections().size(); count++) {
            //Add the emissions from each section together.
            emissions += getSections().get(count)
                    .calculateEmissions(areaBurned);
        }
        return emissions;
    }

    @Override
    public String toString() {
        return name + String.format(" (%.2f kg CO2)", getTotalEmissions());
    }

    public Section getSection(SectionCategory section) {
        if (section == SectionCategory.EXTERNAL_WALLS) {
            return externalWalls;
        } else if (section == SectionCategory.FLOORING) {
            return flooring;
        } else if (section == SectionCategory.ROOFING) {
            return roofing;
        } else {
            return null;
        }
    }

    public ArrayList<Section> getSections() {
        ArrayList<Section> sectionList = new ArrayList<Section>();
        sectionList.add(externalWalls);
        sectionList.add(roofing);
        sectionList.add(flooring);
        return sectionList;
    }

}
