/**
 * *****************************************************************************
 * This application has been revised as a part of an Interactive Qualifying
 * Project for Worcester Polytechnic Institute and our sponsor, SHPI Ingenieria.
 *
 * Revision Developer: Kyle Fortin
 *
 * Revision Authors: Stella Banou, Devin Duarte, Kyle Fortin, Taylor Llodra
 *
 * The application is based on work completed by a previous IQP team from WPI.
 *
 * Original Developer: Tyler Stone
 *
 * Original Authors: Alexandra Hardin, Eric Schattschneider, Tyler Stone,
 * Kristina Walker
 * ****************************************************************************
 */
package sa.shpi.advanced.pdf;

import java.io.FileOutputStream;
import java.util.ArrayList;
import java.util.Date;

import sa.shpi.advanced.model.Industrial;
import sa.shpi.advanced.model.content.FurnishingList;
import sa.shpi.advanced.model.content.Room;
import sa.shpi.advanced.model.structure.MaterialList;

import com.itextpdf.text.Document;
import com.itextpdf.text.DocumentException;
import com.itextpdf.text.Element;
import com.itextpdf.text.Font;
import com.itextpdf.text.Paragraph;
import com.itextpdf.text.Phrase;
import com.itextpdf.text.pdf.PdfPCell;
import com.itextpdf.text.pdf.PdfPTable;
import com.itextpdf.text.pdf.PdfWriter;
import java.awt.Desktop;
import java.io.File;
import java.io.IOException;
import java.time.Instant;
import sa.shpi.advanced.model.content.RoomType;

/**
 * This class automatically generates a PDF report upon instantiation.
 *
 * @author Kyle
 *
 */
public class PdfGenerator {

    private static long date = new Date().getTime();
    /**
     * First text string denotes the file path to be used. Second text string
     * denotes file extention.
     */
    private static String FILE = System.getProperty("user.home") + "/Documents/SHPI Reportes de Emisiones/" + date + ".pdf";
    private static File dir = new File(System.getProperty("user.home") + "/Documents/SHPI Reportes de Emisiones");
    private static File pdf = new File(FILE);
    private static Font catFont = new Font(Font.FontFamily.TIMES_ROMAN, 18,
            Font.BOLD);
    private static Font subFont = new Font(Font.FontFamily.TIMES_ROMAN, 16,
            Font.BOLD);
    private static Font smallBold = new Font(Font.FontFamily.TIMES_ROMAN, 12,
            Font.BOLD);

    private static Industrial res;

    public PdfGenerator(Industrial res) {
        PdfGenerator.res = res;
        
        if (dir.exists()) {
            try {
                Document document = new Document();
                PdfWriter.getInstance(document, new FileOutputStream(FILE));
                document.open();
                addTitlePage(document);
                addContent(document);
                document.close();
                if (Desktop.isDesktopSupported()) {
                    try {
                        Desktop.getDesktop().open(pdf);
                    } catch (IOException ex) {
                        //No application registered for .PDF ...
                    }
                }
            } catch (Exception e) {
                e.printStackTrace();
            }
        } else {
            try {
                dir.mkdir();
                Document document = new Document();
                PdfWriter.getInstance(document, new FileOutputStream(FILE));
                document.open();
                addTitlePage(document);
                addContent(document);
                document.close();
                if (Desktop.isDesktopSupported()) {
                    try {
                        Desktop.getDesktop().open(pdf);
                    } catch (IOException ex) {
                        //No application registered for .PDF ...
                    }
                }
            } catch (Exception e) {
                e.printStackTrace();
            }
        }
    }

    private static void addTitlePage(Document document)
            throws DocumentException {

        Paragraph preface = new Paragraph();
        addEmptyLine(preface, 1);

        // Write the header.
        preface.add(new Paragraph(
                "Reporte de Emisiones - Edificio Industrial",
                catFont));
        preface.add(new Paragraph("SHPI Ingenieria S.A.", smallBold));
        addEmptyLine(preface, 1);

        preface.add(new Paragraph("Reporte generado: " + new Date(), smallBold));
        addEmptyLine(preface, 3);

        document.add(preface);
    }

    private static void addContent(Document document) throws DocumentException {
        //Basic details of fire.
        Paragraph resDetails = new Paragraph("Detalles del incendio:", subFont);
        resDetails.add(new Paragraph("Área total del edificio industrial: " + res.getAreaTotal()
                + " m^2"));
        resDetails.add(new Paragraph("Área quemada: " + res.getAreaBurned()
                + " m^2"));
        resDetails.add(new Paragraph("Categoría de edificio industrial: " + res.getIndustrialType()));
        addEmptyLine(resDetails, 2);

        Paragraph totalEmissions = new Paragraph("Total de Emisiones: "
                + String.format("%.2f kg CO2", res.getTotalEmissions()),
                catFont);

        //Add to document:
        document.add(resDetails);
        document.add(generateStructureSection());
        document.add(generateContentSection());
        document.add(totalEmissions);
    }

    private static Paragraph generateStructureSection() {
        //Add structural details:
        Paragraph structureDetails = new Paragraph("La Estructura ("
                + String.format("%.2f kg CO2", res.getStructureEmissions())
                + "):", subFont);
        PdfPTable structureTable = new PdfPTable(3);

        PdfPCell c1 = new PdfPCell(new Phrase("Categoría"));
        c1.setHorizontalAlignment(Element.ALIGN_CENTER);
        structureTable.addCell(c1);

        c1 = new PdfPCell(new Phrase("Emisiones de Categoría"));
        c1.setHorizontalAlignment(Element.ALIGN_CENTER);
        structureTable.addCell(c1);

        c1 = new PdfPCell(new Phrase("El Material"));
        c1.setHorizontalAlignment(Element.ALIGN_CENTER);
        structureTable.addCell(c1);

        //Complete table dynamically:
        ArrayList<sa.shpi.advanced.model.structure.Section> resSections = res
                .getSections();
        for (int count = 0; count < resSections.size(); count++) {
            structureTable.addCell(resSections.get(count).getSectionCategory()
                    .toString());
            
            structureTable.addCell(String.format("%.2f", resSections.get(count).calculateEmissions(Industrial.getAreaBurned())));

            MaterialList matList = resSections.get(count).getMaterialList();
            for (int c2 = 0; c2 < matList.size(); c2++) {
                structureTable.addCell(matList.get(c2).getMaterial().getName());
                structureTable.addCell("");
                structureTable.addCell("");
            }
            //Add blank cells to complete row:
            structureTable.addCell("");
            //Delete empty row:
            structureTable.deleteLastRow();
        }

        structureDetails.add(structureTable);

        return structureDetails;
    }

    private static Paragraph generateContentSection() {
        //Add conent details:
        Paragraph contentDetails = new Paragraph("Los Contenidos ("
                + String.format("%.2f kg CO2", res.getContentEmissions())
                + "):", subFont);
        PdfPTable contentTable = new PdfPTable(5);

        PdfPCell c1 = new PdfPCell(new Phrase("Área"));
        c1.setHorizontalAlignment(Element.ALIGN_CENTER);
        contentTable.addCell(c1);
        
        c1 = new PdfPCell(new Phrase("Cantidad de Área"));
        c1.setHorizontalAlignment(Element.ALIGN_CENTER);
        contentTable.addCell(c1);
        
        c1 = new PdfPCell(new Phrase("Emisiones de Área"));
        c1.setHorizontalAlignment(Element.ALIGN_CENTER);
        contentTable.addCell(c1);

        c1 = new PdfPCell(new Phrase("Muebles"));
        c1.setHorizontalAlignment(Element.ALIGN_CENTER);
        contentTable.addCell(c1);

        c1 = new PdfPCell(new Phrase("Cantidad de Muebles"));
        c1.setHorizontalAlignment(Element.ALIGN_CENTER);
        contentTable.addCell(c1);

        ArrayList<Room> rooms = res.getRooms();
        for (int count = 0; count < rooms.size(); count++) {
            contentTable.addCell(rooms.get(count).getRoomType().toString());
            
            contentTable.addCell(String.valueOf(rooms.get(count).getRoomQuantity()));
            
            contentTable.addCell(String.format("%.2f", rooms.get(count).calculateEmissions()));

            if (rooms.get(count).getRoomType().equals(RoomType.BATHROOM)) {
                FurnishingList furnishingList = rooms.get(count).getFurnishingList();
                if (furnishingList.size() > 1) {
                    for (int c2 = 0; c2 < furnishingList.size(); c2++) {
                        contentTable.addCell(furnishingList.get(c2).getFurnishing()
                                .getName());
                        contentTable.addCell(String.valueOf(furnishingList.get(c2)
                                .getQuantity()));
                        contentTable.addCell("");
                        contentTable.addCell("");
                        contentTable.addCell("");
                    }
                    contentTable.addCell("");
                    contentTable.addCell("");
                } else {
                    for (int c2 = 0; c2 < furnishingList.size(); c2++) {
                        contentTable.addCell(furnishingList.get(c2).getFurnishing()
                            .getName());
                        contentTable.addCell(String.valueOf(furnishingList.get(c2)
                            .getQuantity()));
                    }
                }
            } else if (rooms.get(count).getRoomType().equals(RoomType.OFFICE)) {
                FurnishingList furnishingList = rooms.get(count).getFurnishingList();
                if (furnishingList.size() > 1) {
                    for (int c2 = 0; c2 < furnishingList.size(); c2++) {
                        contentTable.addCell(furnishingList.get(c2).getFurnishing()
                                .getName());
                        contentTable.addCell(String.valueOf(furnishingList.get(c2)
                                .getQuantity()));
                        contentTable.addCell("");
                        contentTable.addCell("");
                        contentTable.addCell("");
                    }
                    contentTable.addCell("");
                    contentTable.addCell("");
                } else {
                    for (int c2 = 0; c2 < furnishingList.size(); c2++) {
                        contentTable.addCell(furnishingList.get(c2).getFurnishing()
                            .getName());
                        contentTable.addCell(String.valueOf(furnishingList.get(c2)
                            .getQuantity()));
                    }
                }
            } else if (rooms.get(count).getRoomType().equals(RoomType.CONF_ROOM)) {
                FurnishingList furnishingList = rooms.get(count).getFurnishingList();
                if (furnishingList.size() > 1) {
                    for (int c2 = 0; c2 < furnishingList.size(); c2++) {
                        contentTable.addCell(furnishingList.get(c2).getFurnishing()
                                .getName());
                        contentTable.addCell(String.valueOf(furnishingList.get(c2)
                                .getQuantity()));
                        contentTable.addCell("");
                        contentTable.addCell("");
                        contentTable.addCell("");
                    }
                    contentTable.addCell("");
                    contentTable.addCell("");
                } else {
                    for (int c2 = 0; c2 < furnishingList.size(); c2++) {
                        contentTable.addCell(furnishingList.get(c2).getFurnishing()
                            .getName());
                        contentTable.addCell(String.valueOf(furnishingList.get(c2)
                            .getQuantity()));
                    }
                }
            } else if (rooms.get(count).getRoomType().equals(RoomType.BREAK_ROOM)) {
                FurnishingList furnishingList = rooms.get(count).getFurnishingList();
                if (furnishingList.size() > 1) {
                    for (int c2 = 0; c2 < furnishingList.size(); c2++) {
                        contentTable.addCell(furnishingList.get(c2).getFurnishing()
                                .getName());
                        contentTable.addCell(String.valueOf(furnishingList.get(c2)
                                .getQuantity()));
                        contentTable.addCell("");
                        contentTable.addCell("");
                        contentTable.addCell("");
                    }
                    contentTable.addCell("");
                    contentTable.addCell("");
                } else {
                    for (int c2 = 0; c2 < furnishingList.size(); c2++) {
                        contentTable.addCell(furnishingList.get(c2).getFurnishing()
                            .getName());
                        contentTable.addCell(String.valueOf(furnishingList.get(c2)
                            .getQuantity()));
                    }
                }
            } else if (rooms.get(count).getRoomType().equals(RoomType.FIN_GOOD_STORE)) {
                contentTable.addCell(rooms.get(count).getFinGoodType().toString());
                contentTable.addCell(String.format("%.2f", rooms.get(count).getMatMass()) + " kg");
            } else if (rooms.get(count).getRoomType().equals(RoomType.RAW_MAT_STORAGE)) {
                contentTable.addCell(rooms.get(count).getRawMatType().toString());
                contentTable.addCell(String.format("%.2f", rooms.get(count).getRawMatMass()) + " kg");
            } else if (rooms.get(count).getRoomType().equals(RoomType.FUEL_STORE)) {
                contentTable.addCell(rooms.get(count).getFuelType().toString());
                contentTable.addCell(String.format("%.2f", rooms.get(count).getFuelVolume()) + " L");
            } else if (rooms.get(count).getRoomType().equals(RoomType.PROD_FLOOR)) {
                contentTable.addCell(rooms.get(count).getRawMatType().toString());
                contentTable.addCell(String.format("%.2f", rooms.get(count).getRawMatMass()) + " kg");
                contentTable.addCell("");
                contentTable.addCell("");
                contentTable.addCell("");
                contentTable.addCell(rooms.get(count).getFinGoodType().toString());
                contentTable.addCell(String.format("%.2f", rooms.get(count).getMatMass()) + " kg");
            }
            //Add blank cells to complete the row.
            
            //Delete empty row.
            //contentTable.deleteLastRow();
        }

        contentDetails.add(contentTable);

        return contentDetails;
    }

    private static void addEmptyLine(Paragraph paragraph, int number) {
        for (int i = 0; i < number; i++) {
            paragraph.add(new Paragraph(" "));
        }
    }

}
